# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.image.v1 import _proxy as _proxy_v1
from openstack.image.v2 import _proxy as _proxy_v2
from openstack import service_description


class ImageService(
    service_description.ServiceDescription[_proxy_v1.Proxy | _proxy_v2.Proxy]
):
    """The image service."""

    supported_versions = {
        '1': _proxy_v1.Proxy,
        '2': _proxy_v2.Proxy,
    }
